-- ----------------------------------------------------------------------------
-- Miner's Coffee and Pick
-- 		by Nasapunk88
-- ----------------------------------------------------------------------------
local _, L = ...;
MCAP = {};
MCAP.debug = false;
MCAP.isDeleting = false;
MCAP.enableSwap = false;

MCAP.items = {
	{ ["name"] = "MCAP_CoffeeButton",  --"Miner's Coffee"
		["id"] = 118897, 
		["button"] = nil
	},
	{ ["name"] = "MCAP_PickButton", --"Preserved Mining Pick"
		["id"] = 118903,
		["button"] = nil
	}
};

if MCAP.debug then
	table.insert(MCAP.items,{ ["name"] = "MCAP_MilkButton", --Goats milk for testing
			["id"] = 35954,
			["button"] = nil
		});
end

function MCAP:updateButtons()
	self.previous = 0;
	for i = 1, #self.items do
		self:updateButton(self.items[i].button,self.items[i].id,i);
	end
end

function MCAP:updateButton(btn,id,num)
	local count = GetItemCount(id);
	
	local itemName = GetItemInfo(id);
	if itemName == nil then return end;
	
	name, _, buff = AuraUtil.FindAuraByName(itemName,"player");
	--If player is missing buff
	if name == nil or (id == 118897 and buff < MinersCoffeeAndPickCharacterDB.buff ) then
		btn:Show();
		local start, duration = GetItemCooldown(id);
		btn.cooldown:SetCooldown(start, duration);
		if count > 0 then
			btn:ClearAllPoints();
			if MinersCoffeeAndPickDB.alignment == "LEFT" then
				if self.previous == 0 then
					btn:SetPoint("LEFT", self.frame, "LEFT", 0, 0);
				else
					btn:SetPoint("LEFT", self.items[self.previous].button, "RIGHT", 2, 0);
				end
			else
				if self.previous == 0 then
					btn:SetPoint("RIGHT", self.frame, "RIGHT", 0, 0);
				else
					btn:SetPoint("RIGHT", self.items[self.previous].button, "LEFT", -2, 0);
				end
			end
			self.previous = num;
			btn.countString:SetText(format("%d",count));
			if buff and buff > 0 then
				btn.buffString:SetText(format("|cff00ff00%d",buff));
			else
				btn.buffString:SetText("");
			end
			btn.texture:SetDesaturated(false);
		elseif count == 0 then
			btn.countString:SetText("");
			btn.texture:SetDesaturated(true);
			btn:Hide();
		end
	else	--Buff is applied
		btn:Hide();
	end
end

function MCAP:createButton(btn,id)
	btn:Hide();
	btn:SetWidth(38);
	btn:SetHeight(38);
	btn:ClearAllPoints();
	btn:SetClampedToScreen(true);
	--Right click to drag
	btn:EnableMouse(true);
	btn:RegisterForDrag("RightButton");
	btn:SetMovable(true);
	btn:SetScript("OnDragStart", function(self) self:GetParent():StartMoving(); end);
	btn:SetScript("OnDragStop", function(self) 
			self:GetParent():StopMovingOrSizing();
			self:GetParent():SetUserPlaced(false);
			local point, relativeTo, relativePoint, xOfs, yOfs = self:GetParent():GetPoint();
			MinersCoffeeAndPickDB.position = {point, nil, relativePoint, xOfs, yOfs};
		end);
	--Setup macro
	btn:SetAttribute("type", "macro");
	btn:SetAttribute("macrotext", format("/use item:%d",id));
	btn.countString = btn:CreateFontString(btn:GetName().."Count", "OVERLAY", "NumberFontNormal");
	btn.countString:SetPoint("BOTTOMRIGHT", btn, -3, 2);
	btn.countString:SetJustifyH("RIGHT");
	btn.buffString = btn:CreateFontString(btn:GetName().."Buff", "OVERLAY", "NumberFontNormal");
	btn.buffString:SetPoint("BOTTOMLEFT", btn, 2, 2);
	btn.buffString:SetJustifyH("LEFT");
	btn.icon = btn:CreateTexture(nil,"BACKGROUND");
	btn.icon:SetTexture(GetItemIcon(id));
	btn.icon:SetAllPoints(btn);
	btn.texture = btn.icon;
	btn.texture:SetPoint("TOPLEFT", btn, -2, 1);
	btn.cooldown = CreateFrame("Cooldown",btn:GetName().."Cooldown", btn, "CooldownFrameTemplate");
	btn.cooldown:SetAllPoints();
	btn.cooldown:Show();
	--Tooltip
	btn:SetScript("OnEnter", function(self)
		GameTooltip:SetOwner(self,"ANCHOR_TOP");
		GameTooltip:SetItemByID(format("%d",id));
		GameTooltip:SetClampedToScreen(true);
		GameTooltip:Show();
	  end);
	btn:SetScript("OnLeave",GameTooltip_Hide);
 end
 
function MCAP:reset()
	MCAP.frame:ClearAllPoints();
	MCAP.frame:SetPoint('CENTER', UIParent, 'CENTER', 0, 0);
	MinersCoffeeAndPickDB.position = {"CENTER","CENTER",0,0};
	MinersCoffeeAndPickDB.alignment = "LEFT";
	MinersCoffeeAndPickCharacterDB = {
		["keep"] = { ["118897"] = -1,   -- Keep X Miner's Coffee
					 ["118903"] = -1    -- Keep X Preserved Mining Pick
		},
		["buff"] = 5,   -- Buff up to X Miner's Coffee before hiding
		["enable"] = true,   -- Show button
	};
	if MCAP.debug then
		MinersCoffeeAndPickCharacterDB.keep["35954"] = 48;
	end
	MCAP:OnEvent("UPDATE");
end
 
 
function MCAP:OnEvent(event, ...)
	if event == "PLAYER_LOGIN" then
		self.frame:UnregisterEvent("PLAYER_LOGIN");
		MinersCoffeeAndPickDB = MinersCoffeeAndPickDB or {};
		MinersCoffeeAndPickCharacterDB = MinersCoffeeAndPickCharacterDB or {};
		--If DB is empty
		if next (MinersCoffeeAndPickCharacterDB) == nil then
			MCAP:reset();
		end
		if MCAP.debug and MinersCoffeeAndPickCharacterDB.keep["35954"] == nil then
			MinersCoffeeAndPickCharacterDB.keep["35954"] = 48;
		elseif not MCAP.debug and MinersCoffeeAndPickCharacterDB.keep["35954"] then
			MinersCoffeeAndPickCharacterDB.keep["35954"] = nil;
		end
		return
	
	end
	--Check for combat to avoid tainting frame
	if UnitAffectingCombat("player") then
		return
	end
	--Only load buttons when player is in their Mine
	--Get localized minimap zone text
	MCAP.buildingID, MCAP.minimap = C_Garrison.GetBuildingInfo(61);
	local zone = GetMinimapZoneText();
	if zone == L["Alliance Mine"] or zone == L["Horde Mine"]
		or zone == MCAP.minimap
		or strlower(zone) == strlower(MCAP.minimap) then
		self.frame:RegisterEvent("LOOT_CLOSED");
		self.frame:RegisterEvent("UNIT_AURA"); 
		self.frame:RegisterEvent("BAG_UPDATE"); 
		--self.frame:RegisterEvent("ITEM_LOCK_CHANGED");
		MCAP.unregisterEvents = true;
		if MinersCoffeeAndPickCharacterDB.enable  then
			self.frame:Show();
		else
			self.frame:Hide();
		end
		self:updateButtons();
	else --Hide anywhere else
		if MCAP.unregisterEvents then
			self.frame:Hide();
			self.frame:UnregisterEvent("LOOT_CLOSED");
			self.frame:UnregisterEvent("UNIT_AURA");
			self.frame:UnregisterEvent("BAG_UPDATE");
			--self.frame:UnregisterEvent("ITEM_LOCK_CHANGED");
			MCAP.unregisterEvents = false;
		end
	end
end

------------------------------------------------
-- Slash Commands
------------------------------------------------
local function slashHandler(msg)
	msg = msg:lower() or "";
	local command, rest = msg:match("^(%S*)%s*(.-)$");

	if (command == "buff") then
		local num = tonumber(rest);
		if num ~= nil then
			if num > 5 then
				num = 5;
			end
			if num < 1 then
				num = 1;
			end
			MinersCoffeeAndPickCharacterDB.buff = num;
			print("|cffffff78MCAP|r: Using |cff33ff99"..MinersCoffeeAndPickCharacterDB.buff.."|r stacks of "..select(2,GetItemInfo(118897)));
			MCAP:updateButtons();
		else
			print("|cffffff78MCAP|r: Invalid Number Entered: "..rest);
		end
	elseif (msg == "show" or msg == "enable") then
		MinersCoffeeAndPickCharacterDB.enable = true;
		MCAP:OnEvent("UPDATE");
		print("|cffffff78MCAP|r: Enabled");
	elseif (msg == "hide" or msg == "disable") then
		MinersCoffeeAndPickCharacterDB.enable = false;
		MCAP:OnEvent("UPDATE");
		print("|cffffff78MCAP|r: Disabled");
	elseif (msg == "left") then
		MinersCoffeeAndPickDB.alignment = "LEFT";
		MCAP:updateButtons();
		print("|cffffff78MCAP|r: Buttons aligned to the left");
	elseif (msg == "right") then
		MinersCoffeeAndPickDB.alignment = "RIGHT";
		MCAP:updateButtons();
		print("|cffffff78MCAP|r: Buttons aligned to the right");
	elseif (msg == "reset") then
		MCAP:reset();
		print("|cffffff78MCAP|r: Reset");
	else
		print("|cffffff78MCAP|r: Using |cff33ff99"..MinersCoffeeAndPickCharacterDB.buff.."|r stacks of "..select(2,GetItemInfo(118897)));
		print("|cffffff78Miner's Coffee and Pick|r: Arguments to |cffffff78/mcap|r :");
		print("  |cffffff78 show||hide|r - Toggle the buttons.");
		print("  |cffffff78 left||right|r - Set button alignment.");
		print("  |cffffff78 reset|r - Reset all settings.");
		print("  |cffffff78 buff number|r - The number of stacks for "..select(2,GetItemInfo(118897))..".");
		--print("  |cffffff78 coffee number||all|r - The number of "..select(2,GetItemInfo(118897)).." to keep.");
		--print("  |cffffff78 pick number||all|r - The number of "..select(2,GetItemInfo(118903)).." to keep.");
	end
end

SlashCmdList.MinersCoffeAndPick = function(msg) slashHandler(msg) end;
SLASH_MinersCoffeAndPick1 = "/mcap";
SLASH_MinersCoffeAndPick2 = "/minerscoffeeandpick";

--Helper functions
local function cout(msg, premsg)
	premsg = premsg or "[".."Miner's Coffee and Pick".."]"
	print("|cFFE8A317"..premsg.."|r "..msg);
end

local function coutBool(msg,bool)
	if bool then
		print(msg..": true");
	else
		print(msg..": false");
	end
end

--Main Frame
MCAP.frame = CreateFrame("Frame", "MCAP_Frame", UIParent);
MCAP.frame:Hide();
MCAP.frame:SetWidth(76);
MCAP.frame:SetHeight(39);
MCAP.frame:SetClampedToScreen(true);
MCAP.frame:SetFrameStrata("BACKGROUND");
MCAP.frame:SetMovable(true);
MCAP.frame:RegisterEvent("ZONE_CHANGED"); 
MCAP.frame:RegisterEvent("ZONE_CHANGED_INDOORS");
MCAP.frame:RegisterEvent("ZONE_CHANGED_NEW_AREA");
MCAP.frame:RegisterEvent("PLAYER_ENTERING_WORLD");
MCAP.frame:RegisterEvent("PLAYER_REGEN_ENABLED");
MCAP.frame:RegisterEvent("PLAYER_LOGIN");
MCAP.frame:SetScript("OnEvent", function(self,event,...) MCAP:OnEvent(event,...) end);
MCAP.frame:SetScript("OnShow", function(self,event,...) 
	--Restore position
	MCAP.frame:ClearAllPoints();
	if MinersCoffeeAndPickDB and MinersCoffeeAndPickDB.position then
		MCAP.frame:SetPoint(MinersCoffeeAndPickDB.position[1],UIParent,MinersCoffeeAndPickDB.position[3],MinersCoffeeAndPickDB.position[4],MinersCoffeeAndPickDB.position[5]);
	else
		MCAP.frame:SetPoint('CENTER', UIParent, 'CENTER', 0, 0);
	end
 end);
---Create button for each item
for i = 1, #MCAP.items do
	MCAP.items[i].button = CreateFrame("Button", MCAP.items[i].name, MCAP.frame, "SecureActionButtonTemplate,ActionButtonTemplate");
	MCAP:createButton(MCAP.items[i].button,MCAP.items[i].id);
end

------------------------------------------------
-- Interface Options
------------------------------------------------

function MCAP:interfaceOptions_OnShow()
	if not (MCAP.Interface.showButtons == nil) then
		MCAP.Interface.showButtons:SetChecked(MinersCoffeeAndPickCharacterDB.enable);
	end
	MCAP.Interface.buffSlider:SetValue(MinersCoffeeAndPickCharacterDB.buff);
	MCAP_alignmentDropDownText:SetText("Button Alignment");
end

function MCAP:interfaceOptions_Okay()
	MCAP:updateButtons();
	--MCAP:OnEvent("UPDATE");
end

function MCAP:interfaceOptions_Cancel()
	--Settings save as they are changed automatically
end

function MCAP:interfaceOptions_Default()
	MCAP:reset();
end

function MCAP:setupInterfaceOptions()
	MCAP.Interface = {};
	MCAP.InterfacePanel = CreateFrame("Frame", "MCAP_InterfacePanel", InterfaceOptionsFramePanelContainer);
	MCAP.InterfacePanel.name = "Miner's Coffee and Pick";
	MCAP.InterfacePanel:Hide();
	MCAP.InterfacePanel.okay = function() MCAP:interfaceOptions_Okay() end
	MCAP.InterfacePanel.cancel = function() MCAP:interfaceOptions_Cancel() end
	MCAP.InterfacePanel.default = function() MCAP:interfaceOptions_Default() end
	MCAP.InterfacePanel:SetScript("OnShow", function(self,event,...) MCAP:interfaceOptions_OnShow(event,...) end);
	
	--Title
	MCAP.Interface.ConfigTitle = MCAP.InterfacePanel:CreateFontString("MCAP_ConfigTitle", "ARTWORK", "GameFontNormalLarge");
    MCAP.Interface.ConfigTitle:SetPoint("TOPLEFT", 16, -16);
    MCAP.Interface.ConfigTitle:SetText("Miner's Coffee and Pick");
	
	--Description
	MCAP.Interface.ConfigSubtitle = MCAP.InterfacePanel:CreateFontString("MCAP_ConfigSubtitle", "ARTWORK", "GameFontHighlight");
    MCAP.Interface.ConfigSubtitle:SetHeight(22);
    MCAP.Interface.ConfigSubtitle:SetPoint("TOPLEFT", MCAP_ConfigTitle, "BOTTOMLEFT", 0, -8);
    MCAP.Interface.ConfigSubtitle:SetPoint("RIGHT", MCAP_InterfacePanel, -32, 0);
    MCAP.Interface.ConfigSubtitle:SetNonSpaceWrap(false);
	MCAP.Interface.ConfigSubtitle:SetWordWrap(true);
    MCAP.Interface.ConfigSubtitle:SetJustifyH("LEFT");
    MCAP.Interface.ConfigSubtitle:SetJustifyV("TOP");
    MCAP.Interface.ConfigSubtitle:SetText("Display Miner's Coffee and Preserved Mining Pick buttons inside your garrison mine.");
	
	--Show Buttons
	MCAP.Interface.showButtons = CreateFrame("CheckButton", "MCAP_showButtons", MCAP.InterfacePanel, "InterfaceOptionsCheckButtonTemplate");
    MCAP.Interface.showButtons:SetPoint("TOPLEFT", MCAP_ConfigSubtitle, "BOTTOMLEFT", 16, -6);
	_G[MCAP.Interface.showButtons:GetName().."Text"]:SetText("Show Buttons");
	MCAP.Interface.showButtons:SetScript("OnClick", function(self, button, down)  
		PlaySound(self:GetChecked() and 856 or 857);
		MinersCoffeeAndPickCharacterDB.enable = self:GetChecked();
		MCAP:OnEvent("UPDATE");
		end);
	MCAP.Interface.showButtons.tooltipText = "Show Buttons";
	MCAP.Interface.showButtons.tooltipRequirement = "Show MCAP buttons";
	
	--Buff Slider
	MCAP.Interface.buffSlider = CreateFrame("Slider", "MCAP_buffSlider", MCAP.InterfacePanel, "OptionsSliderTemplate");
	MCAP.Interface.buffSlider:SetWidth(150);
	MCAP.Interface.buffSlider:SetHeight(15);
	MCAP.Interface.buffSlider:SetOrientation('HORIZONTAL');
	MCAP.Interface.buffSlider:SetPoint("TOPLEFT", MCAP_showButtons, "BOTTOMLEFT", 0, -25);
	MCAP.Interface.buffSlider:SetMinMaxValues(1, 5);
	MCAP.Interface.buffSlider.minValue, MCAP.Interface.buffSlider.maxValue = MCAP.Interface.buffSlider:GetMinMaxValues();
	_G[MCAP.Interface.buffSlider:GetName() .. 'Low']:SetText(MCAP.Interface.buffSlider.minValue);
	_G[MCAP.Interface.buffSlider:GetName() .. 'High']:SetText(MCAP.Interface.buffSlider.maxValue);
	_G[MCAP.Interface.buffSlider:GetName() .. 'Text']:SetText('Buff Stack Size');
	MCAP.Interface.buffSlider:SetValueStep(1);
	MCAP.Interface.buffSlider:SetObeyStepOnDrag(true);
	MCAP.Interface.buffSlider:SetScript("OnValueChanged", function(self,value)
			MCAP.Interface.buffEditBox:SetNumber(value);
			MinersCoffeeAndPickCharacterDB.buff = value;
			end);
			
	--Buff Box
	MCAP.Interface.buffEditBox = CreateFrame("EditBox", "MCAP_buffEditBox", MCAP.InterfacePanel, "InputBoxTemplate")
	MCAP.Interface.buffEditBox:SetSize(25, 7);
	MCAP.Interface.buffEditBox:SetPoint("CENTER", MCAP_buffSlider, "CENTER", 0, -22);
	MCAP.Interface.buffEditBox:SetAutoFocus(false);
	MCAP.Interface.buffEditBox:SetNumeric(true);
	MCAP.Interface.buffEditBox:SetJustifyH("CENTER");
	MCAP.Interface.buffEditBox:SetJustifyV("CENTER");
	local filename, fontHeight, flags = MCAP.Interface.buffEditBox:GetFont();
	MCAP.Interface.buffEditBox:SetFont(filename,12,flags);
	MCAP.Interface.buffEditBox:SetScript("OnTextChanged", function(self,userInput)
			if self:GetNumber() > 0 and self:GetNumber() < 6 then
				MinersCoffeeAndPickCharacterDB.buff = self:GetNumber();
				MCAP.Interface.buffSlider:SetValue(MinersCoffeeAndPickCharacterDB.buff);
				MCAP:updateButtons();
			end
			end);

			
	--Alignment
	local info = {};
	MCAP.Interface.alignmentDropDown = CreateFrame("Frame", "MCAP_alignmentDropDown", MCAP.InterfacePanel, "UIDropDownMenuTemplate");
	MCAP.Interface.alignmentDropDown:SetPoint("TOPLEFT", MCAP_buffSlider, "BOTTOMLEFT", -18, -40);
	UIDropDownMenu_SetWidth(MCAP.Interface.alignmentDropDown,140,1);
	MCAP.Interface.alignmentDropDown.initialize = function()
		wipe(info);
		local alignments = {"LEFT", "RIGHT"};
		for i, align in next, alignments do
			info.text = alignments[i];
			info.value = alignments[i];
			info.func = function(self)
				MinersCoffeeAndPickDB.alignment = self:GetText();
				MCAP_alignmentDropDownText:SetText(self:GetText());
				MCAP:updateButtons();
			end
			info.checked = align == MinersCoffeeAndPickDB.alignment;
			UIDropDownMenu_AddButton(info);
		end
	end
	MCAP_alignmentDropDownText:SetText("Button Alignment");
	
	--Reset Settings
	MCAP.Interface.resetUI = CreateFrame("Button", "MCAP_resetUI", MCAP.InterfacePanel, "UIPanelButtonTemplate");
	MCAP.Interface.resetUI:SetText("Reset Settings and Position");
	MCAP.Interface.resetUI:SetWidth(178);
	MCAP.Interface.resetUI:SetHeight(24);
	MCAP.Interface.resetUI:SetPoint("TOPLEFT", MCAP_alignmentDropDown, "BOTTOMLEFT",17,-50);
	MCAP.Interface.resetUI:SetScript("OnClick", function()
		MCAP:reset();
	end);
	MCAP.Interface.resetUI.tooltipText = "Resets all settings to default values";
	
	--Note:
	-- MCAP.Interface.note = MCAP.InterfacePanel:CreateFontString("MCAP_note", "ARTWORK", "GameFontHighlight");
    -- MCAP.Interface.note:SetHeight(18);
    -- MCAP.Interface.note:SetPoint("TOPLEFT", MCAP_resetUI, "BOTTOMLEFT", 0, -50);
    -- MCAP.Interface.note:SetPoint("RIGHT", MCAP_InterfacePanel, -32, 0);
    -- MCAP.Interface.note:SetNonSpaceWrap(false);
	-- MCAP.Interface.note:SetWordWrap(true);
    -- MCAP.Interface.note:SetJustifyH("LEFT");
    -- MCAP.Interface.note:SetJustifyV("TOP");
    -- MCAP.Interface.note:SetText("Press the OKAY button below to save changes.");
	
	InterfaceOptions_AddCategory(MCAP.InterfacePanel);
end

function MCAP:openInterfaceOptions()
	InterfaceOptionsFrame_OpenToCategory(MCAP.InterfacePanel);
end


MCAP:setupInterfaceOptions();
